/*
 * Copyright 2013 Andrea "andynaz" Agnesse
 * 
 * This file is part of "iRex DR1000 Merger".
 * 
 * "iRex DR1000 Merger" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * "iRex DR1000 Merger" is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * "iRex DR1000 Merger".  If not, see <http://www.gnu.org/licenses/>.
 * 
 * author e-mail: andynaz@gmail.com
 * author home page: http://andynaz.altervista.org
 * program official page: http://andynaz.altervista.org/Progetti/iRexDR1000Merger.shtm
 */

package it.andynaz.irex.dr1000;

import static java.lang.System.out;

import java.io.*;
import java.util.*;
import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.*;
import it.andynaz.irex.dr1000.scribble.*;


/**
 * Represent a DR1000 PDF file.
 * 
 * An object of that kind is associated with a path to a PDF file.
 *
 * @author andynaz
 * @version 2013-01-16
 */
public class DR1000PdfDoc {
	private File file;
//	private Map<Integer, Page> pageMap;

	private DR1000PdfDoc(File file){
		this.file = file;
	}
	
	/**
	 * Creates a DR1000PdfDoc object.
	 * 
	 * @param filePath path to the PDF file
	 * @return a new DR1000PdfFile object
	 * 
	 * @throws IllegalArgumentException if filePath is not a PDF file
	 */
	public static DR1000PdfDoc getInstance(String filePath){
		if( !filePath.toLowerCase().endsWith(".pdf") )
			throw new IllegalArgumentException("error: no PDF file, filePath="+filePath);
		File file = new File(filePath);
		if( !file.isFile() )
			throw new IllegalArgumentException("error: filePath is not a file, filePath="+filePath);
		return new DR1000PdfDoc(file);
	}

	/**
	 * Return the name of the PDF file.
	 *
	 * @return the name of the PDF file
	 */
	public String getFileName(){
		return this.file.getName();
	}

	/**
	 * Return the PDF file.
	 *
	 * @return the PDF file
	 */
	public File getFile(){
		return this.file;
	}

	/**
	 * Merge the scribbles with the PDF.
	 * 
	 * The output is written in the file specified by the 'output' argument.
	 * 
	 * @param outFilePath the output file
	 * 
	 * @throws Exception in case of error
	 */
	void merge(String outFilePath) throws Exception{
		ScribblesMgr sm = new ScribblesMgr();
		sm.init(file.getParentFile());

		List<Integer> scribbledPages = sm.getScribbledPages(file.getName());
      
		PdfReader reader = new PdfReader(file.getPath());
		PdfStamper stamp = new PdfStamper(reader, new FileOutputStream(outFilePath));
		for( int pageNum=1; pageNum<=reader.getNumberOfPages(); pageNum++ ){
			if( scribbledPages.contains(pageNum) ){
				List<Integer> blob = sm.getBlobScribbles(pageNum, file.getName());
				Page p = createSCribblePage(blob);
				PdfContentByte cbOver = stamp.getOverContent(pageNum);
			//	Rectangle rectPage = reader.getPageSizeWithRotation(pageNum);
				Rectangle rectCropBox = reader.getCropBox(pageNum);
				int nRotation = reader.getPageRotation(pageNum);
				while (nRotation > 0) {
					rectCropBox = rectCropBox.rotate();
					nRotation -= 90;
				}
				if( p!=null )
					p.drawStrokes(cbOver, rectCropBox.left(), rectCropBox.top() /*- rectCropBox.bottom()*/);
			}
		}
		stamp.close();
		sm.closeConnection();
	}

	/**
	 * Create a report on the PDF document.
	 *
	 * The report is printed on the System.out.
	 *
	 * @throws Exception
	 * 
	 * @deprecated only for debug pourpose, will be removed in the next releases
	 */
	void report(String outputFile) throws Exception{
		PdfReader reader = new PdfReader(file.getPath());
		
		int totPage = reader.getNumberOfPages();

		PrintWriter w = null;
		try{
			w = new PrintWriter(outputFile);
			w.println("num page="+totPage);
			for( int pageNum=1; pageNum<=totPage; pageNum++ ){

				w.println("page="+pageNum);
				Rectangle r;
				r = reader.getPageSizeWithRotation(pageNum);
				w.format("\tt=%-8sl=%-8sb=%-8sr=%-8s\n", r.top(), r.left(), r.bottom(), r.right());
				w.format("\tt=%-8sl=%-8sb=%-8sr=%-8s\n", r.getBorderWidthTop(), r.getBorderWidthLeft(),
						  r.getBorderWidthBottom(), r.getBorderWidthRight());

				r = reader.getCropBox(pageNum);
				w.format("\tt=%-8sl=%-8sb=%-8sr=%-8s\n", r.top(), r.left(), r.bottom(), r.right());
				w.format("\tt=%-8sl=%-8sb=%-8sr=%-8s\n", r.getBorderWidthTop(), r.getBorderWidthLeft(),
						  r.getBorderWidthBottom(), r.getBorderWidthRight());
				w.flush();
			}
			w.close();
		} catch(IOException e){
			e.printStackTrace();
		}
	}

	/**
	 * Create a {@link Page} from the blob.
	 * 
	 * This method creates also the {@link Stroke} objects inside the Page.
	 * 
	 * @param l the blob
	 * @return a Page with all the scribbles in the blob, or <tt>null</tt> if the
	 * list 'l' is empty
	 */
	private Page createSCribblePage(List<Integer> l){
		Page p = null;
		if( l==null || l.isEmpty() ) return p;

		p = new Page();
		Util.skip(l, 36);																									//36: useless things
		int nStroke = Util.getIntFromBytes(l.remove(0), l.remove(0), l.remove(0), l.remove(0));	//4: number of strokes
		// ----- stroke -----																							// ----- stroke -----
		for( int c=0; c<nStroke; c++ ){
		//	out.println("\tstroke: "+c);
			Stroke s = new Stroke();
			Util.skip(l, 4);																								//4: color
			float zoom = Util.get4ByteFloat(l.remove(0), l.remove(0), l.remove(0), l.remove(0));	//4: zoom
			s.setZoom(zoom);
			Util.skip(l, 4);																								//4: layer
			Util.skip(l, 4);																								//4: shape
			int width = Util.getIntFromBytes(l.remove(0), l.remove(0), l.remove(0), l.remove(0));	//4: width
			s.setWidth(width);

			int nPoint = Util.getIntFromBytes(l.remove(0), l.remove(0), l.remove(0), l.remove(0));	//4: number of points
		//	out.println("\t\tnum points: "+nPoint);
			// ----- point -----																							// ----- point -----
			for( int j=0; j<nPoint; j++ ){
				assert l.size()>=12 : "size="+l.size();
				int x = Util.getIntFromBytes(l.remove(0), l.remove(0), l.remove(0), l.remove(0));	//4: x
				int y = Util.getIntFromBytes(l.remove(0), l.remove(0), l.remove(0), l.remove(0));	//4: y
		//		out.println("\t\tpoints: "+x+"-"+y);
				s.addPoint(x, y);
				Util.skip(l, 4);																							//4: pression(?)
			}
			p.addStroke(s);
		}

		assert l.isEmpty() : "lista non finita";

		return p;
	}


	/**
	 * Create a file with all the scribbles in a text format.
	 *
	 * @param outputFile path to the output text file
	 * @throws Exception in caso of exception
	 *
	 * @deprecated only for debug purpuse; will be deleted in future release
	 */
	public void format(String outputFile) throws Exception{
		ScribblesMgr sm = new ScribblesMgr();
		sm.init(file.getParentFile());


		List<Integer> pages = sm.getScribbledPages(file.getName());
		Collections.sort(pages);

		BufferedWriter w = null;
		try{
			w = new BufferedWriter(new FileWriter(outputFile));
			w.write("file="+file.getName()); w.newLine();

			for( Integer page : pages ){
				w.write("page="+page); w.newLine();
				List<Integer> l = sm.getBlobScribbles(page, this.getFileName());
				if(l==null) continue;
				// ----- page -----																								// ----- page -----
				w.write(Util.getIntFromBytes(l.remove(0), l.remove(0))+"\n");                                 //2: version
				w.write(Util.getIntFromBytes(l.remove(0), l.remove(0), l.remove(0), l.remove(0))+"\n");       //4: orientation
				w.write(Util.getIntFromBytes(l.remove(0), l.remove(0), l.remove(0), l.remove(0),
						l.remove(0), l.remove(0), l.remove(0), l.remove(0))+"\n");                              //8: page pixel
				w.write(Util.getIntFromBytes(l.remove(0), l.remove(0))+"\n");                                 //2: color R
				w.write(Util.getIntFromBytes(l.remove(0), l.remove(0))+"\n");                                 //2: color G
				w.write(Util.getIntFromBytes(l.remove(0), l.remove(0))+"\n");                                 //2: color B
				w.write(Util.getIntFromBytes(l.remove(0), l.remove(0), l.remove(0), l.remove(0))+"\n");       //4: ul_x
				w.write(Util.getIntFromBytes(l.remove(0), l.remove(0), l.remove(0), l.remove(0))+"\n");       //4: ul_y
				w.write(Util.getIntFromBytes(l.remove(0), l.remove(0), l.remove(0), l.remove(0))+"\n");       //4: lr_x
				w.write(Util.getIntFromBytes(l.remove(0), l.remove(0), l.remove(0), l.remove(0))+"\n");	       //4: lr_y
				                                                                                       // ________________________
				                                                                                       //      36: useless things

				int nStroke = Util.getIntFromBytes(l.remove(0), l.remove(0), l.remove(0), l.remove(0));	       //4: number of strokes
				w.write(nStroke+"\n");

				// ----- stroke -----																							// ----- stroke -----
				for( int c=0; c<nStroke; c++ ){
					String t = "\t";
					w.write(t+Util.getIntFromBytes(l.remove(0), l.remove(0), l.remove(0), l.remove(0))+"\n");  //4: color
					w.write(t+Util.get4ByteFloat(l.remove(0), l.remove(0), l.remove(0), l.remove(0))+"\n");	    //4: zoom
					w.write(t+Util.getIntFromBytes(l.remove(0), l.remove(0), l.remove(0), l.remove(0))+"\n");  //4: layer
					w.write(t+Util.getIntFromBytes(l.remove(0), l.remove(0), l.remove(0), l.remove(0))+"\n");  //4: shape
					w.write(t+Util.getIntFromBytes(l.remove(0), l.remove(0), l.remove(0), l.remove(0))+"\n");  //4: width

					int nPoint = Util.getIntFromBytes(l.remove(0), l.remove(0), l.remove(0), l.remove(0));     //4: number of points
					w.write(t+nPoint+"\n");

					// ----- point -----																							// ----- point -----
					for( int j=0; j<nPoint; j++ ){
						int x = Util.getIntFromBytes(l.remove(0), l.remove(0), l.remove(0), l.remove(0));	       //4: x
						int y = Util.getIntFromBytes(l.remove(0), l.remove(0), l.remove(0), l.remove(0));	       //4: y
						int p = Util.getIntFromBytes(l.remove(0), l.remove(0), l.remove(0), l.remove(0));	       //4: pressure(?)
						w.write(t+t+x);
						w.write(t+t+t+y);
						w.write(t+t+t+p);
						w.write("\n");

					}
				}

				if( l.size()>0 )
					out.println("list not ended");
			}

		} catch(Exception e){
			e.printStackTrace();
		} finally{
			w.close();
		}
	}
	
}
