/*
 * Copyright 2013 Andrea "andynaz" Agnesse
 * 
 * This file is part of "iRex DR1000 Merger".
 * 
 * "iRex DR1000 Merger" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * "iRex DR1000 Merger" is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * "iRex DR1000 Merger".  If not, see <http://www.gnu.org/licenses/>.
 * 
 * author e-mail: andynaz@gmail.com
 * author home page: http://andynaz.altervista.org
 * program official page: http://andynaz.altervista.org/Progetti/iRexDR1000Merger.shtm
 */

package it.andynaz.irex.dr1000;

import static java.lang.System.out;
import java.util.*;
import it.andynaz.log.*;
import it.andynaz.utils.Utils;

/**
 * Let you merge a PDF with its scribbles.
 * 
 * <p>To use this program run in a terminal
 * <pre>java -jar iRexDR1000Merger.jar -f <i>path_to_file</i> [-o <i>output_file</i>] [-a <i>action</i>]</pre>
 * Parameter are:
 * <dl>
 *   <dt><b>f</b></dt>
 *   <dd>target PDF file</dd>
 *   <dt><b>o</b></dt>
 *   <dd>output file (optional, if not present, the default one is taken)</dd>
 *   <dt><b>a</b></dt>
 *   <dd>action to be performed (optional, the default one is 'merge')</dd>
 * </dl></p>
 * 
 * <p>Possible actions are:
 * <dl>
 *   <dt><b>merge</b></dt>
 *   <dd>this is the default: merge the scribbles with the PDF in a new PDF file
 *       (default output file is a PDF file in the same folder of the input one
 *       with a "__NOTED" suffix)</dd>
 *   <dt><b>format</b></dt>
 *   <dd>create a text file with the scribbles data formatted (default output
 *       file is a .dat file in the same folder of the input one</dd>
 *   <dt><b>info_PDF</b></dt>
 *   <dd>creates a text file with the misure obtained by the iText library for
 *       each page of the PDF (default output file is a .report.dat file in the
 *       same folder of the input one</dd>
 * </dl></p>
 * 
 * <p>Optionally, you can invoke the program as
 * <pre>java -jar iRexDR1000Merger.jar <i>path_to_file</i></pre>
 * in which you specify only the input PDF file (action is 'merge' and the
 * output file is the default one).</p>
 *
 * @author andynaz
 * @version 2013-01-16
 */
public class IRexDR1000Merger {

   /**
	 * @param args the command line arguments
	 * @throws Exception in case of errors
    */
   public static void main(String[] args) throws Exception {
		printLicence();
		
		Config.init();
		Logger logger = LoggerMgr.getLogger("main");
		logger.log(Arrays.toString(args), Level.FINEST);
		
		// check input
      if( args.length==0 ){
			logger.log("error: no arguments", Level.SEVERE);
			return;
		}
		
		String inputFile = null;
		if( args.length==1 ){
			inputFile = args[0];
		} else{
			inputFile = Utils.getInitParam("f", args);
			if( inputFile==null ){
				logger.log("error: no input file", Level.SEVERE);
				return;
			}
		}
		logger.log("input file: "+inputFile, Level.DEBUG);
		
		String action = Utils.getInitParam("a", args)==null ? "merge" : Utils.getInitParam("a", args);
		logger.log("action: "+action, Level.DEBUG);
		
		String outputFile = Utils.getInitParam("o", args)==null ? getOutputFile(inputFile, action) : Utils.getInitParam("o", args);
		logger.log("output file: "+outputFile, Level.DEBUG);
		
		DR1000PdfDoc doc = DR1000PdfDoc.getInstance(inputFile);
		if( "merge".equals(action)){
			doc.merge(outputFile);
		} else if( "format".equals(action) ){
			doc.format(outputFile);
		} else if( "info_PDF".equals(action) ){
			doc.report(outputFile);
		}
   }
	
	/**
	 * Get the default output file for every action.
	 * 
	 * The base name for the output file is the inputFile without an eventual
	 * extension.
	 * 
	 * @param inputFile the input file
	 * @param action the action to be performed
	 * @return the default output file for the action
	 */
	private static String getOutputFile(String inputFile, String action){
		String outputFile = inputFile.indexOf('.')!=-1 ? inputFile.substring(0, inputFile.lastIndexOf('.')) : inputFile;
		if(        "merge".equals(action))   outputFile += "__NOTED.pdf";
		else if(  "format".equals(action) )  outputFile += ".dat";
		else if( "info_PDF".equals(action) ) outputFile += ".report.dat";
		return outputFile;
	}

	/**
	 * Print the licence advice.
	 */
	private static void printLicence() {
		out.println("--------------------------------------------------------------------");
		out.println(" iRex DR1000 Merger  Copyright (C) 2013  Andrea \"andynaz\" Agnesse");
		out.println();
		out.println(" This program comes with ABSOLUTELY NO WARRANTY.");
		out.println(" This is free software, and you are welcome to redistribute it");
		out.println(" under certain conditions; read the file 'licence.txt' in the ");
		out.println(" distributed binary for details.");
		out.println("--------------------------------------------------------------------");
		out.println();
	}
	
}
