/*
 * Copyright 2013 Andrea "andynaz" Agnesse
 * 
 * This file is part of "iRex DR1000 Merger".
 * 
 * "iRex DR1000 Merger" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * "iRex DR1000 Merger" is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * "iRex DR1000 Merger".  If not, see <http://www.gnu.org/licenses/>.
 * 
 * author e-mail: andynaz@gmail.com
 * author home page: http://andynaz.altervista.org
 * program official page: http://andynaz.altervista.org/Progetti/iRexDR1000Merger.shtm
 */

package it.andynaz.irex.dr1000.scribble;

import it.andynaz.log.Level;
import it.andynaz.log.LoggerMgr;
import static java.lang.System.out;
import java.io.*;
import java.sql.*;
import java.util.*;

/**
 * Class to manage the scribbles.
 *
 * Let other classes access and read the scribbles in the SQLite file.
 *
 * <p>It wraps a connection to the DB, initialized with the <tt>init</tt>
 * methods. Once the connection is established, info on scribbles can be
 * obtained using the various methods.</p>
 *
 * @author andynaz
 * @version 2013-01-16
 */
public class ScribblesMgr {

	/**
	 * Name of the default SQLite database file for PDF documents.
	 */
	public static final String DB_NAME = "metadata.db";

	private Connection con;

	/**
	 * Creates a new ScribbleMgr.
	 */
	public ScribblesMgr(){ }

	/**
	 * Initialize the connection to the SQLite database with the strokes.
	 *
	 * The SQLite database is stored in a file 'metadata.db' in the same folder
	 * of the PDF file.
	 *
	 * @param dirPath path of the folder which contains the PDF and the
	 * metadata.db file
	 *
	 * @throws ClassNotFoundException if the class for the JDBC driver is not
	 * found
	 * @throws SQLException if SQL errors occur
	 */
	public void init(File dirPath) throws ClassNotFoundException, SQLException{
		init(dirPath, DB_NAME);
	}

	/**
	 * Initialize the connection to an SQLite database with strokes.
	 *
	 * The SQLite database is stored in a SQLite file.
	 *
	 * @param dirPath path of the folder which contains the database file
	 * @param fileName name of the database file
	 *
	 * @throws ClassNotFoundException if the class for the JDBC driver is not
	 * found
	 * @throws SQLException if SQL errors occur
	 */
	public void init(File dirPath, String fileName) throws ClassNotFoundException, SQLException{
		Class.forName("org.sqlite.JDBC");
		File dbPath = new File(dirPath, fileName);
		LoggerMgr.getLogger("init").log("fileName="+fileName, Level.DEBUG);
		this.con = DriverManager.getConnection("jdbc:sqlite:"+dbPath.toString());
	}

	/**
	 * Close the connection to the DB.
	 *
	 * @throws SQLException if SQL errors occur
	 */
	public void closeConnection() throws SQLException{
		if( this.con!=null )
			this.con.close();
	}

	/**
	 * Check if a file has scribbles in the database.
	 *
	 * @param fileName name of the file
	 * @return <tt>true</tt> if the file has scribbles, <tt>false</tt> otherwise
	 * 
	 * @throws SQLException
	 */
	public boolean hasScribbles(String fileName) throws SQLException{
		boolean  hasScribbles = false;
		String query = "SELECT COUNT(*) "
				+ " FROM annotations "
				+ " WHERE file_id=(SELECT file_id FROM file_metadata WHERE filename=?)";
		PreparedStatement pst = con.prepareStatement(query);
		pst.setString(1, fileName);
		ResultSet rs = pst.executeQuery();
		if( rs.next() )
			hasScribbles = rs.getInt(1)!=0;
		rs.close();
		pst.close();
		if( !hasScribbles )
			LoggerMgr.getLogger("hasS").log("no scribble found for file '"+fileName+"'", Level.INFO);
		return hasScribbles;
	}

	/**
	 * Get a list of the numbers of the pages with scribbles.
	 *
	 * @param fileName name of the file
	 * @return
	 * @throws SQLException
	 */
	public List<Integer> getScribbledPages(String fileName) throws SQLException{
		List<Integer> l = new ArrayList<Integer>();
		String query = "SELECT file_position "
				  + " FROM annotations "
				  + " WHERE file_id=(SELECT file_id FROM file_metadata WHERE filename=?) "
				  + " ORDER BY file_position";
		PreparedStatement pst = con.prepareStatement(query);
		pst.setString(1, fileName);
		ResultSet rs = pst.executeQuery();
		while( rs.next() )
			l.add(rs.getInt("file_position"));
		rs.close();
		pst.close();
		LoggerMgr.getLogger("getS").log("file '"+fileName+"' has "+l.size()+" scribbled pages", Level.INFO);
		return l;
	}

	/**
	 * Gets a blob with the scribbles as a list of Integer.
	 *
	 * @param pageNumber the number of the page
	 * @param fileName name of the file
	 * @return a list of Integer with the blob
	 *
	 * @throws SQLException if SQL errors occour
	 */
	public List<Integer> getBlobScribbles(int pageNumber, String fileName) throws SQLException{
		List<Integer> l = null;
		String query = "SELECT data "
				  + " FROM annotations "
				  + " WHERE file_id=(SELECT file_id FROM file_metadata WHERE filename=?) "
				  + "   AND file_position=?";
		PreparedStatement pst = con.prepareStatement(query);
		pst.setString(1, fileName);
		pst.setInt(2, pageNumber);
		ResultSet rs = pst.executeQuery();
		if( rs.next() ){
			l = new ArrayList<Integer>();
			byte[] blob = rs.getBytes("data");
			for( byte b : blob ){
				int i = getUnsignedInt(b);
				assert (i>=0 && i<256) : "conversion 'byte --> int' is wrong!";
				l.add(i); //TODO usare un array precostruito di Integer
			}
		}
		rs.close();
		pst.close();
		return l;
	}

	/*
	 * TODO String chooseFileName() - if the filename is not found (presenta nome e numero di pagine scribbacchinate)
	 * SELECT filename, (SELECT count(file_position) FROM annotations WHERE file_id=f.file_id)
		FROM file_metadata f
		ORDER BY filename
	 */

	/**
	 * Convert a byte in an unsigned int.
	 *
	 * The convertion is based on the bits of the byte (they are considered as a
	 * base-2 number).
	 *
	 * @param b a byte
	 * @return the int corresponding to the bits in the byte as a base-2 number
	 */
	private int getUnsignedInt(byte b){
		return b>=0 ? b : 256+b;
	}
	
}
